"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.scan = exports.remove = exports.exists = exports.get = exports.set = void 0;
const binarySearch_1 = require("./binarySearch");
function set(list, item, cmp) {
    const result = (0, binarySearch_1.binarySearch)(list, item, cmp);
    if (result.found !== undefined) {
        // Replace the whole item.
        list.splice(result.found, 1, item);
    }
    else {
        // Insert at missing index.
        list.splice(result.closest, 0, item);
    }
}
exports.set = set;
function get(list, item, cmp) {
    const result = (0, binarySearch_1.binarySearch)(list, item, cmp);
    if (result.found === undefined)
        return;
    return list[result.found];
}
exports.get = get;
function exists(list, item, cmp) {
    const result = (0, binarySearch_1.binarySearch)(list, item, cmp);
    return result.found !== undefined;
}
exports.exists = exists;
function remove(list, item, cmp) {
    let { found } = (0, binarySearch_1.binarySearch)(list, item, cmp);
    if (found !== undefined) {
        // Remove from index.
        return list.splice(found, 1)[0];
    }
}
exports.remove = remove;
function scan(list, args, cmp) {
    const start = args.gte || args.gt;
    const end = args.lte || args.lt;
    if (start !== undefined && end !== undefined && cmp(start, end) > 0) {
        throw new Error("Invalid bounds.");
    }
    // Start at lower bound.
    let i;
    if (args.reverse) {
        if (end === undefined) {
            i = list.length - 1;
        }
        else {
            const result = (0, binarySearch_1.binarySearch)(list, end, cmp);
            if (result.found === undefined) {
                i = result.closest - 1; // i could be -1!
            }
            else {
                if (args.lt)
                    i = result.found - 1;
                else
                    i = result.found;
            }
        }
    }
    else {
        if (start === undefined) {
            i = 0;
        }
        else {
            const result = (0, binarySearch_1.binarySearch)(list, start, cmp);
            if (result.found === undefined) {
                i = result.closest;
            }
            else {
                if (args.gt)
                    i = result.found + 1;
                else
                    i = result.found;
            }
        }
    }
    const results = [];
    while (true) {
        // End of array.
        if (i >= list.length || i < 0) {
            break;
        }
        if (args.limit && results.length >= args.limit) {
            // Limit condition.
            break;
        }
        if (args.reverse) {
            // Lower bound condition.
            const item = list[i];
            if (args.gt) {
                const dir = cmp(args.gt, item);
                if (dir >= 0) {
                    break;
                }
            }
            if (args.gte) {
                const dir = cmp(args.gte, item);
                if (dir > 0) {
                    break;
                }
            }
            results.push(item);
            i -= 1;
        }
        else {
            // Upper bound condition.
            const item = list[i];
            if (args.lt) {
                const dir = cmp(item, args.lt);
                if (dir >= 0) {
                    break;
                }
            }
            if (args.lte) {
                const dir = cmp(item, args.lte);
                if (dir > 0) {
                    break;
                }
            }
            results.push(item);
            i += 1;
        }
    }
    return results;
}
exports.scan = scan;
//# sourceMappingURL=../../src/helpers/sortedList.js.map