"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useAsyncTupleDatabase = void 0;
const react_1 = require("react");
const shallowEqual_1 = require("./helpers/shallowEqual");
const useRerender_1 = require("./helpers/useRerender");
const main_1 = require("./main");
/** Useful for managing UI state for React with a TupleDatabase. */
function useAsyncTupleDatabase(db, fn, args) {
    const rerender = (0, useRerender_1.useRerender)();
    const resultRef = (0, react_1.useRef)(undefined);
    (0, react_1.useEffect)(() => {
        let stopped = false;
        let stop;
        (0, main_1.subscribeQueryAsync)(db, (db) => fn(db, ...args), (result) => {
            if (stopped)
                return;
            if (!(0, shallowEqual_1.shallowEqual)(resultRef.current, result)) {
                resultRef.current = result;
                rerender();
            }
        }).then(({ result, destroy }) => {
            resultRef.current = result;
            if (stopped)
                destroy();
            else
                stop = destroy;
        });
        return () => {
            stopped = true;
            if (stop)
                stop();
        };
    }, [db, fn, ...args]);
    return resultRef.current;
}
exports.useAsyncTupleDatabase = useAsyncTupleDatabase;
//# sourceMappingURL=../src/useAsyncTupleDatabase.js.map